@@ -11,9 +11,12 @@
 # under the License.
 
 import os
+import warnings
 
 from positional import positional
 
+from keystoneauth1 import session
+
 
 class Adapter(object):
     """An instance of a session with local variables.
@@ -43,13 +46,29 @@ class Adapter(object):
     :param logger: A logging object to use for requests that pass through this
                    adapter.
     :type logger: logging.Logger
+    :param dict allow: Extra filters to pass when discovering API versions.
+                       (optional)
+    :param dict additional_headers: Additional headers that should be attached
+                                    to every request passing through the
+                                    adapter. Headers of the same name specified
+                                    per request will take priority.
+    :param str client_name: The name of the client that created the adapter.
+                            This will be used to create the user_agent.
+    :param str client_version: The version of the client that created the
+                               adapter. This will be used to create the
+                               user_agent.
     """
 
+    client_name = None
+    client_version = None
+
     @positional()
     def __init__(self, session, service_type=None, service_name=None,
                  interface=None, region_name=None, endpoint_override=None,
                  version=None, auth=None, user_agent=None,
-                 connect_retries=None, logger=None):
+                 connect_retries=None, logger=None, allow={},
+                 additional_headers=None, client_name=None,
+                 client_version=None):
         # NOTE(jamielennox): when adding new parameters to adapter please also
         # add them to the adapter call in httpclient.HTTPClient.__init__ as
         # well as to load_adapter_from_argparse below if the argument is
@@ -66,6 +85,13 @@ class Adapter(object):
         self.auth = auth
         self.connect_retries = connect_retries
         self.logger = logger
+        self.allow = allow
+        self.additional_headers = additional_headers or {}
+
+        if client_name:
+            self.client_name = client_name
+        if client_version:
+            self.client_version = client_version
 
     def _set_endpoint_filter_kwargs(self, kwargs):
         if self.service_type:
@@ -94,6 +120,24 @@ class Adapter(object):
             kwargs.setdefault('connect_retries', self.connect_retries)
         if self.logger:
             kwargs.setdefault('logger', self.logger)
+        if self.allow:
+            kwargs.setdefault('allow', self.allow)
+
+        if isinstance(self.session, (session.Session, Adapter)):
+            # these things are unsupported by keystoneclient's session so be
+            # careful with them until everyone has transitioned to ksa.
+            # Allowing adapter allows adapter nesting that auth_token does.
+            if self.client_name:
+                kwargs.setdefault('client_name', self.client_name)
+            if self.client_version:
+                kwargs.setdefault('client_version', self.client_version)
+
+        else:
+            warnings.warn('Using keystoneclient sessions has been deprecated. '
+                          'Please update your software to use keystoneauth1.')
+
+        for k, v in self.additional_headers.items():
+            kwargs.setdefault('headers', {}).setdefault(k, v)
 
         return self.session.request(url, method, **kwargs)
 
@@ -189,7 +233,7 @@ class Adapter(object):
 
     @classmethod
     def register_argparse_arguments(cls, parser, service_type=None):
-        """Attach arguments to a given argparse Parser for Adapters
+        """Attach arguments to a given argparse Parser for Adapters.
 
         :param parser: The argparse parser to attach options to.
         :type parser: argparse.ArgumentParser
@@ -238,7 +282,7 @@ class Adapter(object):
 
     @classmethod
     def register_service_argparse_arguments(cls, parser, service_type):
-        """Attach arguments to a given argparse Parser for Adapters
+        """Attach arguments to a given argparse Parser for Adapters.
 
         :param parser: The argparse parser to attach options to.
         :type parser: argparse.ArgumentParser
