@@ -24,6 +24,7 @@ from keystoneauth1.identity.v3 import federation
 _logger = utils.get_logger(__name__)
 
 __all__ = ('OidcAuthorizationCode',
+           'OidcClientCredentials',
            'OidcPassword',
            'OidcAccessToken')
 
@@ -279,7 +280,7 @@ class _OidcBase(federation.FederationBaseAuth):
         :returns: a python dictionary containing the payload to be exchanged
         :rtype: dict
         """
-        raise NotImplemented()
+        raise NotImplementedError()
 
 
 class OidcPassword(_OidcBase):
