@@ -45,6 +45,11 @@ DEFAULT_USER_AGENT = 'keystoneauth1/%s %s %s/%s' % (
     keystoneauth1.__version__, requests.utils.default_user_agent(),
     platform.python_implementation(), platform.python_version())
 
+# NOTE(jamielennox): Clients will likely want to print more than json. Please
+# propose a patch if you have a content type you think is reasonable to print
+# here and we'll add it to the list as required.
+_LOG_CONTENT_TYPES = set(['application/json'])
+
 _logger = utils.get_logger(__name__)
 
 
@@ -134,7 +139,7 @@ def _determine_calling_package():
             # hit the bottom of the frame stack
             break
 
-    return None
+    return ''
 
 
 def _determine_user_agent():
@@ -153,10 +158,10 @@ def _determine_user_agent():
         name = sys.argv[0]
     except IndexError:
         # sys.argv is empty, usually the Python interpreter prevents this.
-        return None
+        return ''
 
     if not name:
-        return None
+        return ''
 
     name = os.path.basename(name)
     if name in ignored:
@@ -207,6 +212,15 @@ class Session(object):
                                     to every request passing through the
                                     session. Headers of the same name specified
                                     per request will take priority.
+    :param str app_name: The name of the application that is creating the
+                         session. This will be used to create the user_agent.
+    :param str app_version: The version of the application creating the
+                            session. This will be used to create the
+                            user_agent.
+    :param list additional_user_agent: A list of tuple of name, version that
+                                       will be added to the user agent. This
+                                       can be used by libraries that are part
+                                       of the communication process.
     """
 
     user_agent = None
@@ -218,7 +232,8 @@ class Session(object):
     @positional(2)
     def __init__(self, auth=None, session=None, original_ip=None, verify=True,
                  cert=None, timeout=None, user_agent=None,
-                 redirect=_DEFAULT_REDIRECT_LIMIT, additional_headers=None):
+                 redirect=_DEFAULT_REDIRECT_LIMIT, additional_headers=None,
+                 app_name=None, app_version=None, additional_user_agent=None):
 
         self.auth = auth
         self.session = _construct_session(session)
@@ -228,19 +243,14 @@ class Session(object):
         self.timeout = None
         self.redirect = redirect
         self.additional_headers = additional_headers or {}
+        self.app_name = app_name
+        self.app_version = app_version
+        self.additional_user_agent = additional_user_agent or []
+        self._determined_user_agent = None
 
         if timeout is not None:
             self.timeout = float(timeout)
 
-        # Per RFC 7231 Section 5.5.3, identifiers in a user-agent should be
-        # ordered by decreasing significance.  If a user sets their product
-        # that value will be used. Otherwise we attempt to derive a useful
-        # product value. The value will be prepended it to the KSA version,
-        # requests version, and then the Python version.
-
-        if user_agent is None:
-            user_agent = _determine_user_agent()
-
         if user_agent is not None:
             self.user_agent = "%s %s" % (user_agent, DEFAULT_USER_AGENT)
 
@@ -280,7 +290,7 @@ class Session(object):
     def _process_header(header):
         """Redact the secure headers to be logged."""
         secure_headers = ('authorization', 'x-auth-token',
-                          'x-subject-token',)
+                          'x-subject-token', 'x-service-token')
         if header[0].lower() in secure_headers:
             token_hasher = hashlib.sha1()
             token_hasher.update(header[1].encode('utf-8'))
@@ -321,7 +331,7 @@ class Session(object):
             string_parts.append(url)
 
         if headers:
-            for header in six.iteritems(headers):
+            for header in headers.items():
                 string_parts.append('-H "%s: %s"'
                                     % self._process_header(header))
         if json:
@@ -349,7 +359,26 @@ class Session(object):
             if not headers:
                 headers = response.headers
             if not text:
-                text = self._remove_service_catalog(response.text)
+                # NOTE(samueldmq): If the response does not provide enough info
+                # about the content type to decide whether it is useful and
+                # safe to log it or not, just do not log the body. Trying to
+                # read the response body anyways may result on reading a long
+                # stream of bytes and getting an unexpected MemoryError. See
+                # bug 1616105 for further details.
+                content_type = response.headers.get('content-type', None)
+
+                # NOTE(lamt): Per [1], the Content-Type header can be of the
+                # form Content-Type := type "/" subtype *[";" parameter]
+                # [1] https://www.w3.org/Protocols/rfc1341/4_Content-Type.html
+                for log_type in _LOG_CONTENT_TYPES:
+                    if content_type is not None and content_type.startswith(
+                            log_type):
+                        text = self._remove_service_catalog(response.text)
+                        break
+                else:
+                    text = ('Omitted, Content-Type is set to %s. Only '
+                            '%s responses have their bodies logged.')
+                    text = text % (content_type, ', '.join(_LOG_CONTENT_TYPES))
         if json:
             text = self._json.encode(json)
 
@@ -358,7 +387,7 @@ class Session(object):
         if status_code:
             string_parts.append('[%s]' % status_code)
         if headers:
-            for header in six.iteritems(headers):
+            for header in headers.items():
                 string_parts.append('%s: %s' % self._process_header(header))
         if text:
             string_parts.append('\nRESP BODY: %s\n' % text)
@@ -371,7 +400,7 @@ class Session(object):
                 endpoint_filter=None, auth=None, requests_auth=None,
                 raise_exc=True, allow_reauth=True, log=True,
                 endpoint_override=None, connect_retries=0, logger=_logger,
-                allow={}, **kwargs):
+                allow={}, client_name=None, client_version=None, **kwargs):
         """Send an HTTP request with the specified characteristics.
 
         Wrapper around `requests.Session.request` to handle tasks such as
@@ -499,14 +528,46 @@ class Session(object):
         elif self.user_agent:
             user_agent = headers.setdefault('User-Agent', self.user_agent)
         else:
-            user_agent = headers.setdefault('User-Agent', DEFAULT_USER_AGENT)
+            # Per RFC 7231 Section 5.5.3, identifiers in a user-agent should be
+            # ordered by decreasing significance.  If a user sets their product
+            # that value will be used. Otherwise we attempt to derive a useful
+            # product value. The value will be prepended it to the KSA version,
+            # requests version, and then the Python version.
+
+            agent = []
+
+            if self.app_name and self.app_version:
+                agent.append('%s/%s' % (self.app_name, self.app_version))
+            elif self.app_name:
+                agent.append(self.app_name)
+
+            for additional in self.additional_user_agent:
+                agent.append('%s/%s' % additional)
+
+            if client_name and client_version:
+                agent.append('%s/%s' % (client_name, client_version))
+            elif client_name:
+                agent.append(client_name)
+
+            if not agent:
+                # NOTE(jamielennox): determine_user_agent will return an empty
+                # string on failure so checking for None will ensure it is only
+                # called once even on failure.
+                if self._determined_user_agent is None:
+                    self._determined_user_agent = _determine_user_agent()
+
+                if self._determined_user_agent:
+                    agent.append(self._determined_user_agent)
+
+            agent.append(DEFAULT_USER_AGENT)
+            user_agent = headers.setdefault('User-Agent', ' '.join(agent))
 
         if self.original_ip:
             headers.setdefault('Forwarded',
                                'for=%s;by=%s' % (self.original_ip, user_agent))
 
         if json is not None:
-            headers['Content-Type'] = 'application/json'
+            headers.setdefault('Content-Type', 'application/json')
             kwargs['data'] = self._json.encode(json)
 
         for k, v in self.additional_headers.items():
@@ -554,6 +615,30 @@ class Session(object):
 
         resp = send(**kwargs)
 
+        # log callee and caller request-id for each api call
+        if log:
+            # service_name should be fetched from endpoint_filter if it is not
+            # present then use service_type as service_name.
+            service_name = None
+            if endpoint_filter:
+                service_name = endpoint_filter.get('service_name')
+                if not service_name:
+                    service_name = endpoint_filter.get('service_type')
+
+            # Nova uses 'x-compute-request-id' and other services like
+            # Glance, Cinder etc are using 'x-openstack-request-id' to store
+            # request-id in the header
+            request_id = (resp.headers.get('x-openstack-request-id') or
+                          resp.headers.get('x-compute-request-id'))
+            if request_id:
+                logger.debug('%(method)s call to %(service_name)s for '
+                             '%(url)s used request id '
+                             '%(response_request_id)s',
+                             {'method': resp.request.method,
+                              'service_name': service_name,
+                              'url': resp.url,
+                              'response_request_id': request_id})
+
         # handle getting a 401 Unauthorized response by invalidating the plugin
         # and then retrying the request. This is only tried once.
         if resp.status_code == 401 and authenticated and allow_reauth:
@@ -594,8 +679,14 @@ class Session(object):
             except requests.exceptions.Timeout:
                 msg = 'Request to %s timed out' % url
                 raise exceptions.ConnectTimeout(msg)
-            except requests.exceptions.ConnectionError:
-                msg = 'Unable to establish connection to %s' % url
+            except requests.exceptions.ConnectionError as e:
+                # NOTE(sdague): urllib3/requests connection error is a
+                # translation of SocketError. However, SocketError
+                # happens for many different reasons, and that low
+                # level message is often really important in figuring
+                # out the difference between network misconfigurations
+                # and firewall blocking.
+                msg = 'Unable to establish connection to %s: %s' % (url, e)
                 raise exceptions.ConnectFailure(msg)
             except requests.exceptions.RequestException as e:
                 msg = 'Unexpected exception for %(url)s: %(error)s' % {
