@@ -253,6 +253,27 @@ class CommonIdentityTests(object):
         self.assertEqual(self.TEST_COMPUTE_ADMIN + '/v2.0', endpoint_v2)
         self.assertEqual(self.TEST_COMPUTE_ADMIN + '/v3', endpoint_v3)
 
+    def test_discovering_when_version_missing(self):
+        # need to construct list this way for relative
+        disc = fixture.DiscoveryList(v2=False, v3=False)
+        disc.add_v2('v2.0')
+
+        self.stub_url('GET', [], base_url=self.TEST_COMPUTE_ADMIN, json=disc)
+
+        a = self.create_auth_plugin()
+        s = session.Session(auth=a)
+
+        endpoint_v2 = s.get_endpoint(service_type='compute',
+                                     interface='admin',
+                                     version=(2, 0))
+
+        endpoint_v3 = s.get_endpoint(service_type='compute',
+                                     interface='admin',
+                                     version=(3, 0))
+
+        self.assertEqual(self.TEST_COMPUTE_ADMIN + '/v2.0', endpoint_v2)
+        self.assertIsNone(endpoint_v3)
+
     def test_asking_for_auth_endpoint_ignores_checks(self):
         a = self.create_auth_plugin()
         s = session.Session(auth=a)
@@ -592,7 +613,7 @@ class GenericAuthPluginTests(utils.TestCase):
 
         self.assertEqual(text, resp.text)
 
-        for k, v in six.iteritems(self.auth.headers):
+        for k, v in self.auth.headers.items():
             self.assertRequestHeaderEqual(k, v)
 
         self.assertIsNone(self.session.get_token())
