@@ -188,12 +188,14 @@ class SessionTests(utils.TestCase):
         in order to redact secure headers while debug is true.
         """
         session = client_session.Session(verify=False)
-        headers = {'HEADERA': 'HEADERVALB'}
+        headers = {'HEADERA': 'HEADERVALB',
+                   'Content-Type': 'application/json'}
         security_headers = {'Authorization': uuid.uuid4().hex,
                             'X-Auth-Token': uuid.uuid4().hex,
-                            'X-Subject-Token': uuid.uuid4().hex, }
-        body = 'BODYRESPONSE'
-        data = 'BODYDATA'
+                            'X-Subject-Token': uuid.uuid4().hex,
+                            'X-Service-Token': uuid.uuid4().hex}
+        body = '{"a": "b"}'
+        data = '{"c": "d"}'
         all_headers = dict(
             itertools.chain(headers.items(), security_headers.items()))
         self.stub_url('POST', text=body, headers=all_headers)
@@ -206,27 +208,99 @@ class SessionTests(utils.TestCase):
         self.assertIn(body, self.logger.output)
         self.assertIn("'%s'" % data, self.logger.output)
 
-        for k, v in six.iteritems(headers):
+        for k, v in headers.items():
             self.assertIn(k, self.logger.output)
             self.assertIn(v, self.logger.output)
 
         # Assert that response headers contains actual values and
         # only debug logs has been masked
-        for k, v in six.iteritems(security_headers):
+        for k, v in security_headers.items():
             self.assertIn('%s: {SHA1}' % k, self.logger.output)
             self.assertEqual(v, resp.headers[k])
             self.assertNotIn(v, self.logger.output)
 
+    def test_session_debug_output_logs_openstack_request_id(self):
+        """Test x-openstack-request-id is logged in debug logs."""
+        def get_response(log=True):
+            session = client_session.Session(verify=False)
+            endpoint_filter = {'service_name': 'Identity'}
+            headers = {'X-OpenStack-Request-Id': 'req-1234'}
+            body = 'BODYRESPONSE'
+            data = 'BODYDATA'
+            all_headers = dict(itertools.chain(headers.items()))
+            self.stub_url('POST', text=body, headers=all_headers)
+            resp = session.post(self.TEST_URL, endpoint_filter=endpoint_filter,
+                                headers=all_headers, data=data, log=log)
+            return resp
+
+        # if log is disabled then request-id is not logged in debug logs
+        resp = get_response(log=False)
+        self.assertEqual(resp.status_code, 200)
+
+        expected_log = ('POST call to Identity for %s used request '
+                        'id req-1234' % self.TEST_URL)
+        self.assertNotIn(expected_log, self.logger.output)
+
+        # if log is enabled then request-id is logged in debug logs
+        resp = get_response()
+        self.assertEqual(resp.status_code, 200)
+        self.assertIn(expected_log, self.logger.output)
+
     def test_logs_failed_output(self):
         """Test that output is logged even for failed requests."""
         session = client_session.Session()
-        body = uuid.uuid4().hex
+        body = {uuid.uuid4().hex: uuid.uuid4().hex}
 
-        self.stub_url('GET', text=body, status_code=400)
+        self.stub_url('GET', json=body, status_code=400,
+                      headers={'Content-Type': 'application/json'})
         resp = session.get(self.TEST_URL, raise_exc=False)
 
         self.assertEqual(resp.status_code, 400)
+        self.assertIn(list(body.keys())[0], self.logger.output)
+        self.assertIn(list(body.values())[0], self.logger.output)
+
+    def test_logging_body_only_for_specified_content_types(self):
+        """Verify response body is only logged in specific content types.
+
+        Response bodies are logged only when the response's Content-Type header
+        is set to application/json. This prevents us to get an unexpected
+        MemoryError when reading arbitrary responses, such as streams.
+        """
+        OMITTED_BODY = ('Omitted, Content-Type is set to %s. Only '
+                        'application/json responses have their bodies logged.')
+        session = client_session.Session(verify=False)
+
+        # Content-Type is not set
+        body = json.dumps({'token': {'id': '...'}})
+        self.stub_url('POST', text=body)
+        session.post(self.TEST_URL)
+        self.assertNotIn(body, self.logger.output)
+        self.assertIn(OMITTED_BODY % None, self.logger.output)
+
+        # Content-Type is set to text/xml
+        body = '<token><id>...</id></token>'
+        self.stub_url('POST', text=body, headers={'Content-Type': 'text/xml'})
+        session.post(self.TEST_URL)
+        self.assertNotIn(body, self.logger.output)
+        self.assertIn(OMITTED_BODY % 'text/xml', self.logger.output)
+
+        # Content-Type is set to application/json
+        body = json.dumps({'token': {'id': '...'}})
+        self.stub_url('POST', text=body,
+                      headers={'Content-Type': 'application/json'})
+        session.post(self.TEST_URL)
         self.assertIn(body, self.logger.output)
+        self.assertNotIn(OMITTED_BODY % 'application/json', self.logger.output)
+
+        # Content-Type is set to application/json; charset=UTF-8
+        body = json.dumps({'token': {'id': '...'}})
+        self.stub_url(
+            'POST', text=body,
+            headers={'Content-Type': 'application/json; charset=UTF-8'})
+        session.post(self.TEST_URL)
+        self.assertIn(body, self.logger.output)
+        self.assertNotIn(OMITTED_BODY % 'application/json; charset=UTF-8',
+                         self.logger.output)
 
     def test_logging_cacerts(self):
         path_to_certs = '/path/to/certs'
@@ -434,6 +508,7 @@ class AuthPlugin(plugin.BaseAuthPlugin):
 class CalledAuthPlugin(plugin.BaseAuthPlugin):
 
     ENDPOINT = 'http://fakeendpoint/'
+    TOKEN = utils.TestCase.TEST_TOKEN
     USER_ID = uuid.uuid4().hex
     PROJECT_ID = uuid.uuid4().hex
 
@@ -448,7 +523,7 @@ class CalledAuthPlugin(plugin.BaseAuthPlugin):
 
     def get_token(self, session):
         self.get_token_called = True
-        return utils.TestCase.TEST_TOKEN
+        return self.TOKEN
 
     def get_endpoint(self, session, **kwargs):
         self.get_endpoint_called = True
@@ -755,22 +830,24 @@ class SessionAuthTests(utils.TestCase):
 
         auth = AuthPlugin()
         sess = client_session.Session(auth=auth)
-        response = uuid.uuid4().hex
+        response = {uuid.uuid4().hex: uuid.uuid4().hex}
 
         self.stub_url('GET',
-                      text=response,
-                      headers={'Content-Type': 'text/html'})
+                      json=response,
+                      headers={'Content-Type': 'application/json'})
 
         resp = sess.get(self.TEST_URL, logger=logger)
 
-        self.assertEqual(response, resp.text)
+        self.assertEqual(response, resp.json())
         output = io.getvalue()
 
         self.assertIn(self.TEST_URL, output)
-        self.assertIn(response, output)
+        self.assertIn(list(response.keys())[0], output)
+        self.assertIn(list(response.values())[0], output)
 
         self.assertNotIn(self.TEST_URL, self.logger.output)
-        self.assertNotIn(response, self.logger.output)
+        self.assertNotIn(list(response.keys())[0], self.logger.output)
+        self.assertNotIn(list(response.values())[0], self.logger.output)
 
 
 class AdapterTest(utils.TestCase):
@@ -787,11 +864,9 @@ class AdapterTest(utils.TestCase):
 
     TEST_URL = CalledAuthPlugin.ENDPOINT
 
-    def _create_loaded_adapter(self):
-        auth = CalledAuthPlugin()
-        sess = client_session.Session()
-        return adapter.Adapter(sess,
-                               auth=auth,
+    def _create_loaded_adapter(self, sess=None, auth=None):
+        return adapter.Adapter(sess or client_session.Session(),
+                               auth=auth or CalledAuthPlugin(),
                                service_type=self.SERVICE_TYPE,
                                service_name=self.SERVICE_NAME,
                                interface=self.INTERFACE,
@@ -951,21 +1026,23 @@ class AdapterTest(utils.TestCase):
         sess = client_session.Session(auth=auth)
         adpt = adapter.Adapter(sess, auth=auth, logger=logger)
 
-        response = uuid.uuid4().hex
+        response = {uuid.uuid4().hex: uuid.uuid4().hex}
 
-        self.stub_url('GET', text=response,
-                      headers={'Content-Type': 'text/html'})
+        self.stub_url('GET', json=response,
+                      headers={'Content-Type': 'application/json'})
 
         resp = adpt.get(self.TEST_URL, logger=logger)
 
-        self.assertEqual(response, resp.text)
+        self.assertEqual(response, resp.json())
         output = io.getvalue()
 
         self.assertIn(self.TEST_URL, output)
-        self.assertIn(response, output)
+        self.assertIn(list(response.keys())[0], output)
+        self.assertIn(list(response.values())[0], output)
 
         self.assertNotIn(self.TEST_URL, self.logger.output)
-        self.assertNotIn(response, self.logger.output)
+        self.assertNotIn(list(response.keys())[0], self.logger.output)
+        self.assertNotIn(list(response.values())[0], self.logger.output)
 
     def test_unknown_connection_error(self):
         self.stub_url('GET', exc=requests.exceptions.RequestException)
@@ -1150,6 +1227,35 @@ class AdapterTest(utils.TestCase):
         self.assertEqual(override_user_agent,
                          self.requests_mock.last_request.headers['User-Agent'])
 
+    def test_nested_adapters(self):
+        text = uuid.uuid4().hex
+        token = uuid.uuid4().hex
+        url = 'http://keystone.example.com/path'
+
+        sess = client_session.Session()
+        auth = CalledAuthPlugin()
+        auth.ENDPOINT = url
+        auth.TOKEN = token
+
+        adap1 = adapter.Adapter(session=sess,
+                                interface='public')
+        adap2 = adapter.Adapter(session=adap1,
+                                service_type='identity',
+                                auth=auth)
+
+        self.requests_mock.get(url + '/test', text=text)
+
+        resp = adap2.get('/test')
+
+        self.assertEqual(text, resp.text)
+        self.assertTrue(auth.get_endpoint_called)
+
+        self.assertEqual('public', auth.endpoint_arguments['interface'])
+        self.assertEqual('identity', auth.endpoint_arguments['service_type'])
+
+        last_token = self.requests_mock.last_request.headers['X-Auth-Token']
+        self.assertEqual(token, last_token)
+
 
 class TCPKeepAliveAdapterTest(utils.TestCase):
 
